﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Windows.Forms;
using Microsoft.Xrm.Sdk;
using Microsoft.Xrm.Sdk.Metadata;
using RoleUpdater.DelegatesHelpers;
using System.Threading;

namespace RoleUpdater.Controls
{
    public partial class PrivilegeLevelSelectionControl : UserControl
    {
        #region Variables

        /// <summary>
        /// List of privilege updates
        /// </summary>
        private readonly List<PrivilegeAction> _actions;

        /// <summary>
        /// Role manager
        /// </summary>
        private readonly RoleManager _rManager;

        /// <summary>
        /// List of entities
        /// </summary>
        private readonly List<EntityMetadata> _entities;

        private readonly UpdateSettings _settings;

        private Thread _fillPrivThread;

        #endregion Variables

        #region Constructor

        /// <summary>
        /// Initializes a new instance of class PrivilegeLevelSelectionControl
        /// </summary>
        public PrivilegeLevelSelectionControl(RoleManager roleManager, List<EntityMetadata> entities, UpdateSettings settings)
        {
            InitializeComponent();

            _actions = new List<PrivilegeAction>();
            _rManager = roleManager;
            _entities = entities;
            _settings = settings;
        }

        #endregion Constructor

        #region Properties

        /// <summary>
        /// Gets the list of privileges to update
        /// </summary>
        public List<PrivilegeAction> Actions
        {
            get { return _actions; }
        } 

        #endregion Properties

        #region Methods

        private void BtnRemoveClick(object sender, EventArgs e)
        {
            var itemsToRemove = listView1.SelectedItems.Cast<ListViewItem>().ToList();
            var actionsToRemove = new List<PrivilegeAction>();

            foreach(ListViewItem item in itemsToRemove)
            {
                var prv = (Entity)item.Tag;

                foreach (var action in _actions.Where(action => action.PrivilegeId == prv.Id))
                {
                    actionsToRemove.Add(action);
                    listView1.Items.Remove(item);
                }

                foreach(var pAction in actionsToRemove)
                {
                    _actions.Remove(pAction);
                }
            }
        }

        private void AddPrivilegeLevel(PrivilegeLevel level, string privilegeLevelName, Entity privilege)
        {
            if (_actions.Any(x => x.PrivilegeId == privilege.Id))
            {
                lblAlreadyAdded.Visible = true;
            }
            else
            {
                lblAlreadyAdded.Visible = false;

                _actions.Add(new PrivilegeAction
                                 {
                                     Level = level,
                                     PrivilegeId = privilege.Id,
                                     PrivilegeName = privilege["name"].ToString()
                                 });

                var item = new ListViewItem(privilege["name"].ToString());
                item.SubItems.Add(privilegeLevelName);
                item.Tag = privilege;
                listView1.Items.Add(item);
            }
        }

        private void BtnAddOrganizationLevelClick(object sender, EventArgs e)
        {
            foreach (var prv in from ListViewItem item in lvPrivileges.SelectedItems select (Entity)item.Tag)
            {
                AddPrivilegeLevel(PrivilegeLevel.Organization, "Organization", prv);
            }
        }

        private void BtnAddUserLevelClick(object sender, EventArgs e)
        {
            foreach (var prv in from ListViewItem item in lvPrivileges.SelectedItems select (Entity)item.Tag)
            {
                AddPrivilegeLevel(PrivilegeLevel.User, "User", prv);
            }
        }

        private void BtnAddBusinessUnitLevelClick(object sender, EventArgs e)
        {
            foreach (var prv in from ListViewItem item in lvPrivileges.SelectedItems select (Entity)item.Tag)
            {
                AddPrivilegeLevel(PrivilegeLevel.BusinessUnit, "Business Unit", prv);
            }
        }

        private void BtnAddParentChildLevelClick(object sender, EventArgs e)
        {
            foreach (var prv in from ListViewItem item in lvPrivileges.SelectedItems select (Entity)item.Tag)
            {
                AddPrivilegeLevel(PrivilegeLevel.ParentChildBusinessUnit, "Parent : Child Business Unit", prv);
            }
        }

        private void AddNoneLevelClick(object sender, EventArgs e)
        {
            foreach (var prv in from ListViewItem item in lvPrivileges.SelectedItems select (Entity)item.Tag)
            {
                AddPrivilegeLevel(PrivilegeLevel.None, "None", prv);
            }
        }

        #endregion Methods

        private void PrivilegeLevelSelectionControlLoad(object sender, EventArgs e)
        {
            lvPrivileges.Columns[lvPrivileges.Columns.Count - 1].Width = lvPrivileges.Width - 24;
            listView1.Columns[listView1.Columns.Count - 1].Width = -2;

            _fillPrivThread = new Thread(DoWork);
            _fillPrivThread.Start();

            if(_settings != null && _settings.Actions.Count > 0)
            {
                foreach(PrivilegeAction pAction in _settings.Actions)
                {
                    var prv = new Entity("privilege") {Id = pAction.PrivilegeId};
                    prv["name"] = pAction.PrivilegeName;

                    AddPrivilegeLevel(pAction.Level, pAction.LevelName, prv);
                }
            }
        }

        void DoWork()
        {
            var filterTerm = textBox1.Text;
            IEnumerable<Entity> privileges = _rManager.Privileges;

            if (filterTerm.Length != 0)
            {
                privileges =
                    _rManager.Privileges.Where(
                        x => x["name"].ToString().ToLower().IndexOf(filterTerm.ToLower(), StringComparison.Ordinal) >= 0);
            }

            foreach (var privilege in privileges)
            {
                var groupName = string.Empty;
                var group = (from entity in _entities
                                 where entity.Privileges.Any(p => p.PrivilegeId == privilege.Id)
                                 select new{entity.LogicalName, entity.DisplayName.UserLocalizedLabel.Label}).FirstOrDefault();
                var entitySchemaName = (from entity in _entities
                                        where entity.Privileges.Any(p => p.Name == privilege["name"].ToString())
                                        select entity.SchemaName).FirstOrDefault();

                if (group == null)
                { 
                    if (privilege["name"].ToString().EndsWith("Entity"))
                        groupName = "Entity";
                    else if (privilege["name"].ToString().EndsWith("Attribute"))
                        groupName = "Attribute";
                    else if (privilege["name"].ToString().EndsWith("Relationship"))
                        groupName = "Relationship";
                    else if (privilege["name"].ToString().EndsWith("OptionSet"))
                        groupName = "OptionSet";
                    else
                        groupName = "_Common";
                }
                else
                {
                    if (group.LogicalName == "customeraddress")
                        groupName =
                            _entities.First(x => x.LogicalName == "account").DisplayName.UserLocalizedLabel.Label;

                    else if (group.LogicalName == "email" || group.LogicalName == "task"
                        || group.LogicalName == "letter" || group.LogicalName == "phonecall"
                        || group.LogicalName == "appointment" || group.LogicalName == "serviceappointment"
                        || group.LogicalName == "campaignresponse" || group.LogicalName == "fax")
                        groupName =
                            _entities.First(x => x.LogicalName == "activitypointer").DisplayName.UserLocalizedLabel.Label;

                    else
                        groupName = group.Label;
                }

                if (ListViewDelegates.GetGroup(lvPrivileges, groupName) == null)
                    ListViewDelegates.AddGroup(lvPrivileges, groupName);

                var item = new ListViewItem
                {
                    Text = privilege["name"].ToString().Remove(0, 3),
                    Tag = privilege,
                    Group =
                        groupName != null
                            ? ListViewDelegates.GetGroup(lvPrivileges, groupName)
                            : ListViewDelegates.GetGroup(lvPrivileges, "_Common")
                };

                if (entitySchemaName != null)
                    item.Text = item.Text.Replace(entitySchemaName, "");

                ListViewDelegates.AddItem(lvPrivileges, item);
            }

            ListViewDelegates.SortGroup(lvPrivileges, true);
            ListViewDelegates.Sort(lvPrivileges, true);
        }

        private void TextBox1TextChanged(object sender, EventArgs e)
        {
            if(_fillPrivThread != null)
                _fillPrivThread.Abort();

            ListViewDelegates.ClearItems(lvPrivileges);
            ListViewDelegates.ClearGroups(lvPrivileges);

            _fillPrivThread = new Thread(DoWork);
            _fillPrivThread.Start();
        }

        private void LvPrivilegesSelectedIndexChanged(object sender, EventArgs e)
        {
            if (lvPrivileges.SelectedItems.Count <= 0) return;

            bool canBeLocal = true;
            bool canBeBasic = true;
            bool canBeDeep = true;
            bool canBeGlobal = true;

            foreach(ListViewItem item in lvPrivileges.SelectedItems)
            {
                var privilege = item.Tag as Entity;

                if (privilege == null) continue;

                if((bool) privilege["canbelocal"] == false)
                    canBeLocal = false;
                 if((bool) privilege["canbebasic"] == false)
                    canBeBasic = false;
                 if((bool) privilege["canbedeep"] == false)
                    canBeDeep = false;
                 if((bool) privilege["canbeglobal"] == false)
                    canBeGlobal = false;
            }

            btnAddUserLevel.Enabled = canBeLocal;
            btnAddBusinessUnitLevel.Enabled = canBeBasic;
            btnAddParentChildLevel.Enabled = canBeDeep;
            btnAddOrganizationLevel.Enabled = canBeGlobal;
        }

        private void ListView1MouseDoubleClick(object sender, MouseEventArgs e)
        {
            if(listView1.SelectedItems.Count > 0)
                listView1.Items.Remove(listView1.SelectedItems[0]);
        }
    }
}
